<?php

namespace Drupal\ui_patterns_layouts\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Layout\LayoutDefinition;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\ui_patterns\ComponentPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides layout plugin definitions for components.
 *
 * @see \Drupal\ui_patterns_layouts\Plugin\Layout\ComponentLayout
 */
class ComponentLayout extends DeriverBase implements ContainerDeriverInterface {

  /**
   * Constructs new ComponentLayout Deriver.
   *
   * @param \Drupal\ui_patterns\ComponentPluginManager $pluginManager
   *   The component plugin manager.
   */
  public function __construct(protected ComponentPluginManager $pluginManager) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('plugin.manager.sdc')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $components = $this->pluginManager->getSortedDefinitions();
    foreach ($components as $component) {
      /** @var \Drupal\Core\Layout\LayoutDefinition $base_plugin_definition */
      $definition = array_merge([
        "deriver" => $base_plugin_definition->getDeriver(),
        "class" => $base_plugin_definition->getClass(),
      ], [
        'label' => $component['annotated_name'] ?? $component['name'] ?? $component['id'],
        'category' => $component['group'] ?? t('Others'),
        'provider' => $component['provider'],
        'id' => $component['id'],
        'context_definitions' => [
          'entity' => new ContextDefinition('entity', 'Entity', FALSE),
        ],
        'admin_label' => $component['annotated_name'] ?? $component['name'] ?? $component['id'],
        // "context_mapping" => ["entity" => "layout_builder.entity"],
        "regions" => [],
      ]);

      if (isset($component['slots']) && is_array($component['slots']) && count($component['slots']) > 0) {
        foreach ($component['slots'] as $slot_id => $slot) {
          $definition['regions'][$slot_id] = ['label' => $slot['title']];
        }
      }
      $id = str_replace('-', '_', (string) $component['id']);
      $this->derivatives[$id] = new LayoutDefinition($definition);
    }
    return $this->derivatives;
  }

}
